package com.caimei.controller;


import com.alibaba.fastjson.JSONObject;
import com.caimei.model.ResponseJson;
import com.caimei.service.OrderService;
import com.caimei.utils.KeyUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Aslee
 */
@Slf4j
@RestController
@RequestMapping("/order")
public class OrderApi {
    private OrderService orderService;

    @Autowired
    public void setOrderService(OrderService orderService) {
        this.orderService = orderService;
    }


    /**
     * 下单
     * data 参数格式：{
     *                     "sign": ****,        //签名
     *                     "notifyUrl": ****,       //订单支付完成通知回调地址
     *                     "addressInfo": {         //收货地址信息
     *                          "receiveMan":"收货人",     //收货人
     *                          "mobile":"15816666666",    //手机号
     *                          "provinceName":"广东省",   //省
     *                          "cityName":"深圳市",       //市/县
     *                          "townName":"福田区",       //区
     *                          "addressDetail":"*****"   //详细地址
     *                     },          //
     *                     "orderId": cm6858,          //对接组织的订单Id，由组织自行生成
     *                     "productInfo": { "0001":2,    // 商品Id:购买数量
     *                                       "0002":3
     *                                   }
     *                     "payInfo": {               // 订单信息
     *                         "orderShouldPayFee": 609.11,       //订单总金额
     *                         "freight": "15",                   //运费金额，到付：0，深圳市内10，省内15
     *                         "freePostFlag": 1                  //-1：到付，1：有运费
     *                     },
     *                     "invoiceInfo": {}// 发票信息： {"type": 0},// 不开发票
     *                                     {
     *                                         "type": 2,             // 要发票
     *                                         "invoiceTitle": "单位名称",
     *                                         "corporationTaxNum": "NSRSBM97897",
     *                                         "registeredAddress": "注册地址",
     *                                         "registeredPhone": "15814011616",
     *                                         "openBank": "开户银行",
     *                                         "bankAccountNo": "987987465465464"
     *                                     }
     *                 }
     */
    @PostMapping("/submit")
    public ResponseJson orderSubmit(HttpServletRequest request) {
        log.info("API对接组织下单");
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空");
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空");
        }
        return orderService.orderSubmit(cmAccount, data);
    }

    /**
     * 订单详情
     */
    @PostMapping("/detail")
    public ResponseJson<Map<String, Object>> orderDetail(HttpServletRequest request) {
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空", null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空", null);
        }
        return orderService.orderDetail(cmAccount, data);
    }

    /**
     * 取消订单
     */
    @PostMapping("/cancel")
    public ResponseJson orderCancel(HttpServletRequest request) {
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空", null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空", null);
        }
        return orderService.orderCancel(cmAccount, data);
    }

    /**
     * 测试通知回调
     */
    @PostMapping("/pay/callBack")
    public ResponseJson callBack(HttpServletRequest request) {
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空", null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空", null);
        }
        return orderService.payCallBack(cmAccount, data);
    }
}
