package com.caimei.controller;

import com.caimei.model.ResponseJson;
import com.caimei.model.vo.ProductDetailVo;
import com.caimei.model.vo.ProductListVo;
import com.caimei.service.ProductService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author Aslee
 */
@Slf4j
@RestController
@RequestMapping("/product")
public class ProductApi {

    private ProductService productService;

    @Autowired
    public void setOrderService(ProductService productService) {
        this.productService = productService;
    }

    /**
     * 商品列表
     * data 参数格式：{
     *     "sign":****,             //签名
     *     "pageNum":1,             //页码
     *     "pageSize":10,           //每页大小
     *     "productName":精华油     //模糊查询的商品名称
     * }
     * return [
     *          {
     *              "productId": "0003",                            //商品Id
     *             "productName": "卡琪紫红色精华油",                //商品名称
     *             "image": "****TH2AZX5L6R20140415014819.png",     //图片路径
     *             "price": 379.75,                                 //价格
     *             "unit": "瓶",                                     //规格
     *             "minBuyNumber": 1,                               //最少购买数量
     *             "ladderPriceFlag": 0,                            //阶梯价格标识：0关闭，1开启
     *             "ladderPriceList": null,                         //阶梯价格列表
     *             "brandName": null                                //品牌名称
     *          }...
     *      ]
     */
    @PostMapping("/list")
    public ResponseJson<List<ProductListVo>> productList(HttpServletRequest request){
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空",null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("商品数据不能为空",null);
        }
        return productService.productList(cmAccount, data);
    }

    /**
     * 商品详情
     */
    @PostMapping("/detail")
    public ResponseJson<ProductDetailVo> productDetail(HttpServletRequest request) {
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空",null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("商品数据不能为空",null);
        }
        return productService.productDetail(cmAccount, data);
    }
}
