package com.caimei.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei.mapper.OrganizeMapper;
import com.caimei.mapper.ProductMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.po.*;
import com.caimei.model.vo.LadderPriceVo;
import com.caimei.model.vo.ProductDetailVo;
import com.caimei.model.vo.ProductListVo;
import com.caimei.model.vo.ProductVo;
import com.caimei.service.ProductService;
import com.caimei.utils.MathUtil;
import com.caimei.utils.ProductUtils;
import com.caimei.utils.KeyUtils;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;

/**
 * @author Aslee
 */
@Slf4j
@Service
public class ProductServiceImpl implements ProductService {
    @Resource
    OrganizeMapper organizeMapper;
    @Resource
    private ProductMapper productMapper;

    @Value("${caimei.oldapi}")
    private String domain;

    @Override
    public ResponseJson<List<ProductListVo>> productList(String cmAccount, String data){
        // 获取组织信息
        CmApiOrganizePo organizePo = organizeMapper.getOrganizeByCmAccount(cmAccount);
        if (null == organizePo) {
            return ResponseJson.error("参数异常，用户不存在", null);
        }
        JSONObject productInfo = null;
        try {
            // 使用组织对应的公钥解密，获得商品数据
            productInfo = KeyUtils.decryptDataPublic(data, organizePo.getPublicKey());
            // 私钥验签
            String preSign = productInfo.getString("sign");
            productInfo.remove("sign");
            String sufSign = KeyUtils.buildSign(productInfo, organizePo.getPrivateKey());
            if (!preSign.equals(sufSign)) {
                return ResponseJson.error("验签失败", null);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseJson.error("解密失败", null);
        }
        Integer pageNum = (Integer) (null == productInfo.get("pageNum") ? 1 : productInfo.get("pageNum"));
        Integer pageSize = (Integer) (null == productInfo.get("pageSize") ? 10 : productInfo.get("pageSize"));
        String productName = (String) productInfo.get("productName");
        PageHelper.startPage(pageNum, pageSize);
        List<ProductVo> productList = productMapper.getProductList(organizePo.getOrganizeId(), productName);
        List<ProductListVo> resultVoList = new ArrayList<>();
        DecimalFormat decimalFormat = new DecimalFormat("0000");
        for (ProductVo product : productList) {
            setProductPrice(product);
            ProductListVo productResult = new ProductListVo();
            productResult.setProductId(decimalFormat.format(product.getProductId()));
            productResult.setProductName(product.getProductName());
            productResult.setShopName(product.getShopName());
            productResult.setImage(product.getMainImage());
            productResult.setPrice(product.getRetailPrice());
            productResult.setBrandName(product.getBrandName());
            productResult.setMinBuyNumber(product.getMinBuyNumber());
            productResult.setUnit(product.getUnit());
            productResult.setLadderPriceFlag(product.getLadderPriceFlag());
            productResult.setLadderPriceList(product.getLadderPriceList());
            resultVoList.add(productResult);
        }
        return ResponseJson.success(resultVoList);
    }


    @Override
    public ResponseJson<ProductDetailVo> productDetail(String cmAccount, String data) {
        // 获取组织信息
        CmApiOrganizePo organizePo = organizeMapper.getOrganizeByCmAccount(cmAccount);
        if (null == organizePo) {
            return ResponseJson.error("参数异常，用户不存在", null);
        }
        JSONObject productInfo = null;
        try {
            // 使用组织对应的公钥解密，获得订单数据
            productInfo = KeyUtils.decryptDataPublic(data, organizePo.getPublicKey());
            // 私钥验签
            String preSign = productInfo.getString("sign");
            productInfo.remove("sign");
            String sufSign = KeyUtils.buildSign(productInfo, organizePo.getPrivateKey());
            if (!preSign.equals(sufSign)) {
                return ResponseJson.error("验签失败", null);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseJson.error("解密失败", null);
        }
        String productId = (String) productInfo.get("productId");
        ProductVo product = productMapper.findProductByProductId(organizePo.getOrganizeId(), productId);
        if (null == product) {
            return ResponseJson.error("商品不存在", null);
        }
        setProductPrice(product);
        ProductDetailVo productDetail = new ProductDetailVo();
        productDetail.setImage(ProductUtils.getImageURL("product", product.getMainImage(), 0, domain));
        //商品图片
        List<ProductImagePo> imageList = productMapper.findProductImagesByOriginalId(product.getOriginalProductId());
        String images = "";
        for (ProductImagePo image : imageList) {
            String imageURL = ProductUtils.getImageURL("product", image.getImage(), 0, domain);
            image.setImage(imageURL);
            images += "".equals(images) ? "" : "," + image.getImage();
        }
        productDetail.setImages(images);
        DecimalFormat decimalFormat = new DecimalFormat("0000");
        //商品详情
        ProductDetailInfoPo detailInfo = productMapper.findProductDetailByOriginalId(product.getOriginalProductId());
        productDetail.setDetailInfo(detailInfo.getDetailInfo());
        productDetail.setDetailInfoText(detailInfo.getDetailInfoText());
        productDetail.setProductId(decimalFormat.format(product.getProductId()));
        productDetail.setProductName(product.getProductName());
        productDetail.setShopName(product.getShopName());
        productDetail.setImage(product.getMainImage());
        productDetail.setPrice(product.getRetailPrice());
        productDetail.setBrandName(product.getBrandName());
        productDetail.setMinBuyNumber(product.getMinBuyNumber());
        productDetail.setUnit(product.getUnit());
        productDetail.setLadderPriceFlag(product.getLadderPriceFlag());
        productDetail.setLadderPriceList(product.getLadderPriceList());
        return ResponseJson.success(productDetail);
    }


    public void setProductPrice(ProductVo product) {
        // 设置图片
        product.setMainImage(ProductUtils.getImageURL("product", product.getMainImage(), 0, domain));
        // 设置商品展示价格
        if (1 == product.getLadderPriceFlag()) {
            // 启用阶梯价格
            getLadderPrice(product);
        } else {
            product.setLadderPriceFlag(0);
            product.setRetailPrice(product.getPrice());
        }
        //添加税费
        boolean addTaxFlag = (0 == product.getIncludedTax() && (1 == product.getInvoiceType() || 2 == product.getInvoiceType()));
        if (addTaxFlag) {
            BigDecimal addedValueTax = MathUtil.div(MathUtil.mul(product.getRetailPrice(), product.getClubTaxPoint()), BigDecimal.valueOf(100));
            BigDecimal price = MathUtil.add(product.getRetailPrice(), addedValueTax);
            product.setRetailPrice(price);
        }
    }


    public void getLadderPrice(ProductVo product) {
        // 启用阶梯价格
        List<LadderPriceVo> ladderPriceList = productMapper.findLadderPriceByProductId(product.getProductId());
        if (ladderPriceList.size() > 0) {
            for (int i = 0; i < ladderPriceList.size(); i++) {
                if (null != ladderPriceList.get(i)) {
                    String buyNumRangeShow;
                    if ((i + 1) < ladderPriceList.size() && null != ladderPriceList.get(i + 1)) {
                        buyNumRangeShow = ladderPriceList.get(i).getBuyNum() + "~" + (ladderPriceList.get(i + 1).getBuyNum() - 1L);
                    } else {
                        buyNumRangeShow = "≥" + ladderPriceList.get(i).getBuyNum();
                    }
                    ladderPriceList.get(i).setBuyNumRangeShow(buyNumRangeShow);
                }
            }
            product.setRetailPrice(ladderPriceList.get(0).getBuyPrice());
            product.setMinBuyNumber(ladderPriceList.get(0).getBuyNum());
            product.setLadderPriceList(ladderPriceList);
            //阶梯价列表添加税费
            boolean addTaxFlag = (0 == product.getIncludedTax() && (1 == product.getInvoiceType() || 2 == product.getInvoiceType()));
            if (addTaxFlag) {
                ladderPriceList.forEach(item -> {
                    item.setBuyPrice(MathUtil.add(item.getBuyPrice(), MathUtil.div(MathUtil.mul(item.getBuyPrice(), product.getClubTaxPoint()), 100)));
                });
            }
        } else {
            // 阶梯价格异常，关闭阶梯价格
            product.setLadderPriceFlag(0);
            product.setRetailPrice(product.getPrice());
        }
    }


}
